package
Version:
v0.8.16
Opens a new window with list of versions in this module.
Published: Mar 17, 2026
License: MIT
Opens a new window with license information.
Imports: 2
Opens a new window with list of imports.
Imported by: 0
Opens a new window with list of known importers.
Documentation
¶
func InspectStruct(s any)
InspectStruct takes any object and prints its fields and values.
Source Files
¶
Click to show internal directories.
Click to hide internal directories.